@@ -1,21 +1,28 @@
 import os
 import re
 import urllib.request
-from datetime import datetime, timedelta
+from datetime import datetime
 from typing import Any, Dict, List, Literal, Optional, Tuple, Union
 
+import numpy as np
 import pandas as pd
 from currency_converter import SINGLE_DAY_ECB_URL, CurrencyConverter
+from numpy.typing import NDArray
 
 from bbstrader.metatrader.utils import (
+    TIMEFRAMES,
     AccountInfo,
     BookInfo,
     InvalidBroker,
     OrderCheckResult,
     OrderSentResult,
+    RateDtype,
+    RateInfo,
     SymbolInfo,
     SymbolType,
     TerminalInfo,
+    TickDtype,
+    TickFlag,
     TickInfo,
     TradeDeal,
     TradeOrder,
@@ -33,6 +40,7 @@ except ImportError:
 __all__ = [
     "Account",
     "Broker",
+    "MetaQuotes",
     "AdmiralMarktsGroup",
     "JustGlobalMarkets",
     "PepperstoneGroupLimited",
@@ -41,7 +49,8 @@ __all__ = [
 ]
 
 __BROKERS__ = {
-    "AMG": "Admirals Group AS",
+    "MQL": "MetaQuotes Ltd.",
+    "AMG": "Admirals SC Ltd",
     "JGM": "Just Global Markets Ltd.",
     "FTMO": "FTMO S.R.O.",
     "PGL": "Pepperstone Group Limited",
@@ -67,14 +76,13 @@ _ADMIRAL_MARKETS_PRODUCTS_ = [
 ]
 _JUST_MARKETS_PRODUCTS_ = ["Stocks", "Crypto", "indices", "Commodities", "Forex"]
 
-SUPPORTED_BROKERS = [__BROKERS__[b] for b in {"AMG", "JGM", "FTMO"}]
+SUPPORTED_BROKERS = [__BROKERS__[b] for b in {"MQL", "AMG", "JGM", "FTMO"}]
 INIT_MSG = (
-    f"\n* Ensure you have a good and stable internet connexion\n"
-    f"* Ensure you have an activete MT5 terminal install on your machine\n"
-    f"* Ensure you have an active MT5 Account with {' or '.join(SUPPORTED_BROKERS)}\n"
-    f"* If you want to trade {', '.join(_ADMIRAL_MARKETS_PRODUCTS_)}, See [{_ADMIRAL_MARKETS_URL_}]\n"
-    f"* If you want to trade {', '.join(_JUST_MARKETS_PRODUCTS_)}, See [{_JUST_MARKETS_URL_}]\n"
-    f"* If you are looking for a prop firm, See [{_FTMO_URL_}]\n"
+    f"\n* Check your internet connection\n"
+    f"* Make sure MT5 is installed and active\n"
+    f"* Looking for a boker? See [{_ADMIRAL_MARKETS_URL_}] "
+    f"or [{_JUST_MARKETS_URL_}]\n"
+    f"* Looking for a prop firm? See [{_FTMO_URL_}]\n"
 )
 
 amg_url = _ADMIRAL_MARKETS_URL_
@@ -136,32 +144,56 @@ AMG_EXCHANGES = {
 }
 
 
-def check_mt5_connection(**kwargs) -> bool:
+def check_mt5_connection(
+    *,
+    path=None,
+    login=None,
+    password=None,
+    server=None,
+    timeout=60_000,
+    portable=False,
+    **kwargs,
+) -> bool:
     """
     Initialize the connection to the MetaTrader 5 terminal.
 
-    Args:
-        path (str, optional): The path to the MetaTrader 5 terminal executable file.
-            Defaults to None (e.g., "C:/Program Files/MetaTrader 5/terminal64.exe").
-        login (int, optional): The login ID of the trading account. Defaults to None.
-        password (str, optional): The password of the trading account. Defaults to None.
-        server (str, optional): The name of the trade server to which the client terminal is connected.
-            Defaults to None.
-        timeout (int, optional): Connection timeout in milliseconds. Defaults to 60_000.
-        portable (bool, optional): If True, the portable mode of the terminal is used.
-            Defaults to False (See https://www.metatrader5.com/en/terminal/help/start_advanced/start#portable).
+    Parameters
+    ----------
+    path : str, optional
+        Path to the MetaTrader 5 terminal executable file.
+        Defaults to ``None`` (e.g., ``"C:/Program Files/MetaTrader 5/terminal64.exe"``).
+    login : int, optional
+        The login ID of the trading account. Defaults to ``None``.
+    password : str, optional
+        The password of the trading account. Defaults to ``None``.
+    server : str, optional
+        The name of the trade server to which the client terminal is connected.
+        Defaults to ``None``.
+    timeout : int, optional
+        Connection timeout in milliseconds. Defaults to ``60_000``.
+    portable : bool, optional
+        If ``True``, the portable mode of the terminal is used.
+        Defaults to ``False``.
+        See: https://www.metatrader5.com/en/terminal/help/start_advanced/start#portable
 
-    Notes:
-        If you want to lunch multiple terminal instances:
-        - Follow these instructions to lunch each terminal in portable mode first:
-            https://www.metatrader5.com/en/terminal/help/start_advanced/start#configuration_file
+    Returns
+    -------
+    bool
+        ``True`` if the connection is successfully established, otherwise ``False``.
+
+    Notes
+    -----
+    If you want to launch multiple terminal instances:
+
+    * First, launch each terminal in **portable mode**.
+    * See instructions: https://www.metatrader5.com/en/terminal/help/start_advanced/start#configuration_file
     """
-    path = kwargs.get("path", None)
-    login = kwargs.get("login", None)
-    password = kwargs.get("password", None)
-    server = kwargs.get("server", None)
-    timeout = kwargs.get("timeout", 60_000)
-    portable = kwargs.get("portable", False)
+
+    if login is not None and server is not None:
+        account_info = mt5.account_info()
+        if account_info is not None:
+            if account_info.login == login and account_info.server == server:
+                return True
 
     init = False
     if path is None and (login or password or server):
@@ -185,9 +217,9 @@ def check_mt5_connection(**kwargs) -> bool:
         else:
             init = mt5.initialize()
         if not init:
-            raise_mt5_error(INIT_MSG)
+            raise_mt5_error(str(mt5.last_error()) + INIT_MSG)
     except Exception:
-        raise_mt5_error(INIT_MSG)
+        raise_mt5_error(str(mt5.last_error()) + INIT_MSG)
     return init
 
 
@@ -221,6 +253,11 @@ class Broker(object):
         return f"{self.__class__.__name__}({self.name})"
 
 
+class MetaQuotes(Broker):
+    def __init__(self, **kwargs):
+        super().__init__(__BROKERS__["MQL"], **kwargs)
+
+
 class AdmiralMarktsGroup(Broker):
     def __init__(self, **kwargs):
         super().__init__(__BROKERS__["AMG"], **kwargs)
@@ -262,6 +299,7 @@ class AMP(Broker): ...
 
 BROKERS: Dict[str, Broker] = {
     "FTMO": FTMO(),
+    "MQL": MetaQuotes(),
     "AMG": AdmiralMarktsGroup(),
     "JGM": JustGlobalMarkets(),
     "PGL": PepperstoneGroupLimited(),
@@ -503,7 +541,7 @@ class Account(object):
         """Helper function to  print account info"""
         self._show_info(self.get_account_info, "account")
 
-    def get_terminal_info(self, show=False) -> Union[TerminalInfo, None]:
+    def get_terminal_info(self, show=False) -> TerminalInfo | None:
         """
         Get the connected MetaTrader 5 client terminal status and settings.
 
@@ -951,7 +989,7 @@ class Account(object):
                         futures_types["bonds"].append(info.name)
             return futures_types[category]
 
-    def get_symbol_info(self, symbol: str) -> Union[SymbolInfo, None]:
+    def get_symbol_info(self, symbol: str) -> SymbolInfo | None:
         """Get symbol properties
 
         Args:
@@ -984,6 +1022,14 @@ class Account(object):
             msg = self._symbol_info_msg(symbol)
             raise_mt5_error(message=f"{str(e)} {msg}")
 
+    def _symbol_info_msg(self, symbol):
+        return (
+            f"No history found for {symbol} in Market Watch.\n"
+            f"* Ensure {symbol} is selected and displayed in the Market Watch window.\n"
+            f"* See https://www.metatrader5.com/en/terminal/help/trading/market_watch\n"
+            f"* Ensure the symbol name is correct.\n"
+        )
+
     def show_symbol_info(self, symbol: str):
         """
         Print symbol properties
@@ -993,15 +1039,7 @@ class Account(object):
         """
         self._show_info(self.get_symbol_info, "symbol", symbol=symbol)
 
-    def _symbol_info_msg(self, symbol):
-        return (
-            f"No history found for {symbol} in Market Watch.\n"
-            f"* Ensure {symbol} is selected and displayed in the Market Watch window.\n"
-            f"* See https://www.metatrader5.com/en/terminal/help/trading/market_watch\n"
-            f"* Ensure the symbol name is correct.\n"
-        )
-
-    def get_tick_info(self, symbol: str) -> Union[TickInfo, None]:
+    def get_tick_info(self, symbol: str) -> TickInfo | None:
         """Get symbol tick properties
 
         Args:
@@ -1043,20 +1081,170 @@ class Account(object):
         """
         self._show_info(self.get_tick_info, "tick", symbol=symbol)
 
-    def get_market_book(self, symbol: str) -> Union[Tuple[BookInfo], None]:
-        """
-        Get the Market Depth content for a specific symbol.
+    def get_rate_info(self, symbol: str, timeframe: str = "1m") -> RateInfo | None:
+        """Get the most recent bar for a specified symbol and timeframe.
+
         Args:
-            symbol (str): Financial instrument name. Required unnamed parameter.
-                The symbol name should be specified in the same format as in the Market Watch window.
-
+            symbol (str): The symbol for which to get the rate information.
+            timeframe (str): The timeframe for the rate information. Default is '1m'.
+                            See ``bbstrader.metatrader.utils.TIMEFRAMES`` for supported timeframes.
         Returns:
-            The Market Depth content as a tuple from BookInfo entries featuring order type, price and volume in lots.
-            Return None in case of an error.
-
+            RateInfo: The most recent bar as a RateInfo named tuple.
+            None: If no rates are found or an error occurs.
         Raises:
             MT5TerminalError: A specific exception based on the error code.
         """
+        rates = mt5.copy_rates_from_pos(symbol, TIMEFRAMES[timeframe], 0, 1)
+        if rates is None or len(rates) == 0:
+            return None
+        rate = rates[0]
+        return RateInfo(*rate)
+
+    def get_rates_from_pos(
+        self, symbol: str, timeframe: str, start_pos: int = 0, bars: int = 1
+    ) -> NDArray[np.void]:
+        """
+        Get bars from the MetaTrader 5 terminal starting from the specified index.
+
+        Args:
+            symbol: Financial instrument name, for example, "EURUSD"
+            timeframe: Timeframe the bars are requested for.
+            start_pos: Initial index of the bar the data are requested from.
+            bars: Number of bars to receive.
+
+        Returns:
+            bars as the numpy array with the named time, open, high, low, close, tick_volume, spread and real_volume columns.
+            Returns an empty array in case of an error.
+        """
+        rates = mt5.copy_rates_from_pos(symbol, TIMEFRAMES[timeframe], start_pos, bars)
+        if rates is None or len(rates) == 0:
+            return np.array([], dtype=RateDtype)
+        return rates
+
+    def get_rates_from_date(
+        self,
+        symbol: str,
+        timeframe: str,
+        date_from: datetime | pd.Timestamp,
+        bars: int = 1,
+    ) -> NDArray[np.void]:
+        """
+        Get bars from the MetaTrader 5 terminal starting from the specified date.
+
+        Args:
+            symbol: Financial instrument name, for example, "EURUSD"
+            timeframe: Timeframe the bars are requested for.
+            date_from: Date of opening of the first bar from the requested sample.
+            bars: Number of bars to receive.
+
+        Returns:
+            bars as the numpy array with the named time, open, high, low, close, tick_volume, spread and real_volume columns.
+            Returns an empty array in case of an error.
+
+        """
+        rates = mt5.copy_rates_from(symbol, TIMEFRAMES[timeframe], date_from, bars)
+        if rates is None or len(rates) == 0:
+            return np.array([], dtype=RateDtype)
+        return rates
+
+    def get_rates_range(
+        self,
+        symbol: str,
+        timeframe: str,
+        date_from: datetime | pd.Timestamp,
+        date_to: datetime | pd.Timestamp = datetime.now(),
+    ) -> NDArray[np.void]:
+        """
+        Get bars in the specified date range from the MetaTrader 5 terminal.
+
+        Args:
+            symbol: Financial instrument name, for example, "EURUSD"
+            timeframe: Timeframe the bars are requested for.
+            date_from: Date the bars are requested from.
+            date_to: Date, up to which the bars are requested.
+
+        Returns:
+            bars as the numpy array with the named time, open, high, low, close, tick_volume, spread and real_volume columns.
+            Returns an empty array in case of an error.
+        """
+        rates = mt5.copy_rates_range(symbol, TIMEFRAMES[timeframe], date_from, date_to)
+        if rates is None or len(rates) == 0:
+            return np.array([], dtype=RateDtype)
+        return rates
+
+    def get_tick_from_date(
+        self,
+        symbol: str,
+        date_from: datetime | pd.Timestamp,
+        ticks: int = 1,
+        flag: Literal["all", "info", "trade"] = "all",
+    ) -> NDArray[np.void]:
+        """
+        Get ticks from the MetaTrader 5 terminal starting from the specified date.
+
+        Args:
+            symbol: Financial instrument name, for example, "EURUSD"
+            date_from: Date the ticks are requested from.
+            ticks: Number of bars to receive.
+            flag: A flag to define the type of the requested ticks ("all", "info", "trade").
+
+        Returns:
+            Returns ticks as the numpy array with the named time, bid, ask, last and flags columns.
+            Return an empty array in case of an error.
+        """
+        ticks_data = mt5.copy_ticks_from(symbol, date_from, ticks, TickFlag[flag])
+        if ticks_data is None or len(ticks_data) == 0:
+            return np.array([], dtype=TickDtype)
+        return ticks_data
+
+    def get_tick_range(
+        self,
+        symbol: str,
+        date_from: datetime | pd.Timestamp,
+        date_to: datetime | pd.Timestamp = datetime.now(),
+        flag: Literal["all", "info", "trade"] = "all",
+    ) -> NDArray[np.void]:
+        """
+        Get ticks for the specified date range from the MetaTrader 5 terminal.
+
+        Args:
+            symbol: Financial instrument name, for example, "EURUSD"
+            date_from: Date the ticks are requested from.
+            date_to: Date, up to which the ticks are requested.
+            flag: A flag to define the type of the requested ticks ("all", "info", "trade").
+
+        Returns:
+            Returns ticks as the numpy array with the named time, bid, ask, last and flags columns.
+            Return an empty array in case of an error.
+        """
+        ticks_data = mt5.copy_ticks_range(symbol, date_from, date_to, TickFlag[flag])
+        if ticks_data is None or len(ticks_data) == 0:
+            return np.array([], dtype=TickDtype)
+        return ticks_data
+
+    def get_market_book(self, symbol: str) -> Tuple[BookInfo]:
+        """
+        Get the Market Depth content for a specific symbol.
+
+        Parameters
+        ----------
+        symbol : str
+            Financial instrument name. The symbol should be specified in the
+            same format as in the Market Watch window.
+
+        Returns
+        -------
+        tuple of BookInfo or None
+            The Market Depth content as a tuple of ``BookInfo`` entries
+            (order type, price, and volume in lots).  
+            Returns ``None`` in case of an error.
+
+        Raises
+        ------
+        MT5TerminalError
+            A specific exception based on the error code.
+        """
+
         try:
             book = mt5.market_book_get(symbol)
             return (
@@ -1185,7 +1373,7 @@ class Account(object):
         group: Optional[str] = None,
         ticket: Optional[int] = None,
         to_df: bool = False,
-    ) -> Union[pd.DataFrame, Tuple[TradePosition], None]:
+    ) -> Union[pd.DataFrame, Tuple[TradePosition]]:
         """
         Get open positions with the ability to filter by symbol or ticket.
         There are four call options:
@@ -1215,7 +1403,6 @@ class Account(object):
         Returns:
             Union[pd.DataFrame, Tuple[TradePosition], None]:
             - `TradePosition` in the form of a named tuple structure (namedtuple) or pd.DataFrame.
-            - `None` in case of an error.
 
         Notes:
             The method allows receiving all open positions within a specified period.
@@ -1271,7 +1458,7 @@ class Account(object):
         position: Optional[int] = None,  # TradePosition.ticket
         to_df: bool = True,
         save: bool = False,
-    ) -> Union[pd.DataFrame, Tuple[TradeDeal], None]:
+    ) -> Union[pd.DataFrame, Tuple[TradeDeal]]:
         """
         Get deals from trading history within the specified interval
         with the ability to filter by `ticket` or `position`.
@@ -1309,7 +1496,6 @@ class Account(object):
         Returns:
             Union[pd.DataFrame, Tuple[TradeDeal], None]:
             - `TradeDeal` in the form of a named tuple structure (namedtuple) or pd.DataFrame().
-            - `None` in case of an error.
 
         Notes:
             The method allows receiving all history orders within a specified period.
@@ -1372,7 +1558,7 @@ class Account(object):
         group: Optional[str] = None,
         ticket: Optional[int] = None,
         to_df: bool = False,
-    ) -> Union[pd.DataFrame, Tuple[TradeOrder], None]:
+    ) -> Union[pd.DataFrame, Tuple[TradeOrder]]:
         """
         Get active orders with the ability to filter by symbol or ticket.
         There are four call options:
@@ -1399,7 +1585,6 @@ class Account(object):
         Returns:
             Union[pd.DataFrame, Tuple[TradeOrder], None]:
             - `TradeOrder` in the form of a named tuple structure (namedtuple) or pd.DataFrame().
-            - `None` in case of an error.
 
         Notes:
             The method allows receiving all history orders within a specified period.
@@ -1464,7 +1649,7 @@ class Account(object):
         position: Optional[int] = None,  # position ticket
         to_df: bool = True,
         save: bool = False,
-    ) -> Union[pd.DataFrame, Tuple[TradeOrder], None]:
+    ) -> Union[pd.DataFrame, Tuple[TradeOrder]]:
         """
         Get orders from trading history within the specified interval
         with the ability to filter by `ticket` or `position`.
@@ -1502,7 +1687,6 @@ class Account(object):
         Returns:
             Union[pd.DataFrame, Tuple[TradeOrder], None]
             - `TradeOrder` in the form of a named tuple structure (namedtuple) or pd.DataFrame().
-            - `None` in case of an error.
 
         Notes:
             The method allows receiving all history orders within a specified period.
@@ -1583,16 +1767,11 @@ class Account(object):
         Returns:
             List[TradeDeal]: List of today deals
         """
-        date_from = datetime.now() - timedelta(days=2)
-        history = (
-            self.get_trades_history(date_from=date_from, group=group, to_df=False) or []
-        )
+        history = self.get_trades_history(group=group, to_df=False) or []
         positions_ids = set([deal.position_id for deal in history if deal.magic == id])
         today_deals = []
         for position in positions_ids:
-            deal = self.get_trades_history(
-                date_from=date_from, position=position, to_df=False
-            )
+            deal = self.get_trades_history(position=position, to_df=False) or []
             if deal is not None and len(deal) == 2:
                 deal_time = datetime.fromtimestamp(deal[1].time)
                 if deal_time.date() == datetime.now().date():
