@@ -2,6 +2,8 @@ from datetime import datetime
 from enum import Enum
 from typing import NamedTuple, Optional
 
+import numpy as np
+
 try:
     import MetaTrader5 as MT5
 except ImportError:
@@ -28,6 +30,10 @@ __all__ = [
     "HistoryNotFound",
     "InvalidVersion",
     "AuthFailed",
+    "RateInfo",
+    "RateDtype",
+    "TickDtype",
+    "TickFlag",
     "UnsupportedMethod",
     "AutoTradingDisabled",
     "InternalFailSend",
@@ -285,6 +291,26 @@ class SymbolType(Enum):
     unknown = "UNKNOWN"  # Unknown or unsupported type
 
 
+TickDtype = np.dtype(
+    [
+        ("time", "<i8"),
+        ("bid", "<f8"),
+        ("ask", "<f8"),
+        ("last", "<f8"),
+        ("volume", "<u8"),
+        ("time_msc", "<i8"),
+        ("flags", "<u4"),
+        ("volume_real", "<f8"),
+    ]
+)
+
+TickFlag = {
+    "all": MT5.COPY_TICKS_ALL,
+    "info": MT5.COPY_TICKS_INFO,
+    "trade": MT5.COPY_TICKS_TRADE,
+}
+
+
 class TickInfo(NamedTuple):
     """
     Represents the last tick for the specified financial instrument.
@@ -308,6 +334,44 @@ class TickInfo(NamedTuple):
     volume_real: float
 
 
+RateDtype = np.dtype(
+    [
+        ("time", "<i8"),
+        ("open", "<f8"),
+        ("high", "<f8"),
+        ("low", "<f8"),
+        ("close", "<f8"),
+        ("tick_volume", "<u8"),
+        ("spread", "<i4"),
+        ("real_volume", "<u8"),
+    ]
+)
+
+
+class RateInfo(NamedTuple):
+    """
+    Reprents a candle (bar) for a specified period.
+    * time: Time in seconds since 1970.01.01 00:00
+    * open: Open price
+    * high: High price
+    * low: Low price
+    * close: Close price
+    * tick_volume: Tick volume
+    * spread: Spread value
+    * real_volume: Real volume
+
+    """
+
+    time: int
+    open: float
+    high: float
+    low: float
+    close: float
+    tick_volume: float
+    spread: int
+    real_volume: float
+
+
 class BookInfo(NamedTuple):
     """
     Represents the structure of a book.
@@ -486,11 +550,7 @@ class MT5TerminalError(Exception):
         self.code = code
         self.message = message
 
-    def __str__(self) -> str:
-        # if self.message is None:
-        #     return f"{self.__class__.__name__}"
-        # else:
-        #     return f"{self.__class__.__name__}, {self.message}"
+    def __repr__(self) -> str:
         msg_str = str(self.message) if self.message is not None else ""
         return f"{self.code} - {self.__class__.__name__}: {msg_str}"
 
@@ -547,7 +607,8 @@ class AutoTradingDisabled(MT5TerminalError):
 class InternalFailError(MT5TerminalError):
     """Base exception class for internal IPC errors."""
 
-    pass
+    def __init__(self, code, message):
+        super().__init__(code, message)
 
 
 class InternalFailSend(InternalFailError):
@@ -640,9 +701,9 @@ def raise_mt5_error(message: Optional[str] = None):
     """
     if message and isinstance(message, Exception):
         message = str(message)
-    error = _ERROR_CODE_TO_EXCEPTION_.get(MT5.last_error()[0])
-    if error is not None:
-        raise Exception(f"{error(None)} {message or MT5.last_error()[1]}")
+    exception = _ERROR_CODE_TO_EXCEPTION_.get(MT5.last_error()[0])
+    if exception is not None:
+        raise exception(f"{message or MT5.last_error()[1]}")
     else:
         raise Exception(f"{message or MT5.last_error()[1]}")
 
