@@ -1,7 +1,10 @@
+import concurrent.futures
+import functools
 import multiprocessing as mp
 import sys
 import time
 from datetime import date, datetime
+from multiprocessing.synchronize import Event
 from typing import Callable, Dict, List, Literal, Optional
 
 import pandas as pd
@@ -42,6 +45,7 @@ MT5_ENGINE_TIMEFRAMES = list(_TF_MAPPING.keys())
 TradingDays = ["monday", "tuesday", "wednesday", "thursday", "friday"]
 WEEK_DAYS = TradingDays + ["saturday", "sunday"]
 FRIDAY = "friday"
+WEEK_ENDS = ["friday", "saturday", "sunday"]
 
 BUYS = ["BMKT", "BLMT", "BSTP", "BSTPLMT"]
 SELLS = ["SMKT", "SLMT", "SSTP", "SSTPLMT"]
@@ -179,6 +183,8 @@ class Mt5ExecutionEngine:
         mm: bool = True,
         auto_trade: bool = True,
         prompt_callback: Callable = None,
+        multithread: bool = False,
+        shutdown_event: Event = None,
         optimizer: str = "equal",
         trail: bool = True,
         stop_trail: Optional[int] = None,
@@ -187,8 +193,8 @@ class Mt5ExecutionEngine:
         show_positions_orders: bool = False,
         iter_time: int | float = 5,
         use_trade_time: bool = True,
-        period: Literal["24/7", "day", "week", "month"] = "week",
-        period_end_action: Literal["break", "sleep"] = "break",
+        period: Literal["24/7", "day", "week", "month"] = "month",
+        period_end_action: Literal["break", "sleep"] = "sleep",
         closing_pnl: Optional[float] = None,
         trading_days: Optional[List[str]] = None,
         comment: Optional[str] = None,
@@ -207,6 +213,10 @@ class Mt5ExecutionEngine:
                 the user for confimation.
             prompt_callback : Callback function to prompt the user for confirmation.
                 This is useful when integrating with GUI applications.
+             multithread : If True, use a thread pool to process signals in parallel.
+                If False, process them sequentially. Set this to True only if the engine
+                is running in a separate process. Default to False.
+            shutdown_event : Use to terminate the copy process when runs in a custum environment like web App or GUI.
             show_positions_orders : Print open positions and orders. Defaults to False.
             iter_time : Interval to check for signals and `mm`. Defaults to 5.
             use_trade_time : Open trades after the time is completed. Defaults to True.
@@ -251,6 +261,7 @@ class Mt5ExecutionEngine:
         self.mm = mm
         self.auto_trade = auto_trade
         self.prompt_callback = prompt_callback
+        self.multithread = multithread
         self.optimizer = optimizer
         self.trail = trail
         self.stop_trail = stop_trail
@@ -274,12 +285,17 @@ class Mt5ExecutionEngine:
 
         self._initialize_engine(**kwargs)
         self.strategy = self._init_strategy(**kwargs)
+        self.shutdown_event = (
+            shutdown_event if shutdown_event is not None else mp.Event()
+        )
         self._running = True
 
     def __repr__(self):
-        trades = self.trades_instances.keys()
+        symbols = self.trades_instances.keys()
         strategy = self.strategy_cls.__name__
-        return f"{self.__class__.__name__}(Symbols={list(trades)}, Strategy={strategy})"
+        return (
+            f"{self.__class__.__name__}(Symbols={list(symbols)}, Strategy={strategy})"
+        )
 
     def _initialize_engine(self, **kwargs):
         global logger
@@ -316,6 +332,7 @@ class Mt5ExecutionEngine:
     def _print_exc(self, msg: str, e: Exception):
         if isinstance(e, KeyboardInterrupt):
             logger.info("Stopping the Execution Engine ...")
+            self.stop()
             sys.exit(0)
         if self.debug_mode:
             raise ValueError(msg).with_traceback(e.__traceback__)
@@ -499,17 +516,12 @@ class Mt5ExecutionEngine:
         logger.info(sessionmsg)
 
     def _check_is_day_ends(self, trade: Trade, symbol, period_type, today, closing):
-        if trade.days_end():
-            self._logmsgif("Day", symbol) if today != FRIDAY else self._logmsgif(
+        if trade.days_end() or (today in WEEK_ENDS and today != FRIDAY):
+            self._logmsgif("Day", symbol) if today not in WEEK_ENDS else self._logmsgif(
                 "Week", symbol
             )
             if (
-                (
-                    period_type == "month"
-                    and today == FRIDAY
-                    and self._is_month_ends()
-                    and closing
-                )
+                (period_type == "month" and self._is_month_ends() and closing)
                 or (period_type == "week" and today == FRIDAY and closing)
                 or (period_type == "day" and closing)
                 or (period_type == "24/7" and closing)
@@ -532,15 +544,17 @@ class Mt5ExecutionEngine:
         self.strategy.perform_period_end_checks()
         if self.period_end_action == "break" and closing:
             sys.exit(0)
-        elif self.period_end_action == "sleep" and today != FRIDAY or not closing:
+        elif (
+            self.period_end_action == "sleep" and today not in WEEK_ENDS
+        ):
             self._sleep_over_night(sessionmsg)
-        elif self.period_end_action == "sleep" and today == FRIDAY:
+        elif self.period_end_action == "sleep" and today in WEEK_ENDS:
             self._sleep_over_weekend(sessionmsg)
 
     def _weekly_end_checks(self, today, closing, sessionmsg):
-        if today != FRIDAY:
+        if today not in WEEK_ENDS:
             self._sleep_over_night(sessionmsg)
-        elif today == FRIDAY:
+        else:
             self.strategy.perform_period_end_checks()
             if self.period_end_action == "break" and closing:
                 sys.exit(0)
@@ -548,9 +562,9 @@ class Mt5ExecutionEngine:
                 self._sleep_over_weekend(sessionmsg)
 
     def _monthly_end_cheks(self, today, closing, sessionmsg):
-        if today != FRIDAY:
+        if today not in WEEK_ENDS and not self._is_month_ends():
             self._sleep_over_night(sessionmsg)
-        elif today == FRIDAY and self._is_month_ends() and closing:
+        elif self._is_month_ends() and closing:
             self.strategy.perform_period_end_checks()
             sys.exit(0)
         else:
@@ -617,12 +631,12 @@ class Mt5ExecutionEngine:
     def _update_risk(self, weights):
         try:
             check_mt5_connection(**self.kwargs)
-            if weights is not None:
+            if weights is not None and not all(v == 0 for v in weights.values()):
+                assert self.daily_risk is not None
                 for symbol in self.symbols:
                     if symbol not in weights:
                         continue
                     trade = self.trades_instances[symbol]
-                    assert self.daily_risk is not None
                     dailydd = round(weights[symbol] * self.daily_risk, 5)
                     trade.dailydd = dailydd
         except Exception as e:
@@ -799,6 +813,15 @@ class Mt5ExecutionEngine:
         signal = {"LONG": "BMKT", "BUY": "BMKT", "SHORT": "SMKT", "SELL": "SMKT"}.get(
             signal, signal
         )
+        if (
+            self.trades_instances[symbol].dailydd == 0
+            and signal not in EXIT_SIGNAL_ACTIONS
+        ):
+            logger.info(
+                f"Daily Risk is set to 0 !!! No trades allowed for SYMBOL={symbol}, "
+                f"STRATEGY={self.STRATEGY} , ACCOUNT={self.ACCOUNT}"
+            )
+            return
         info, sigmsg, msg, tfmsg, riskmsg = self._get_signal_info(
             signal, symbol, price, stoplimit
         )
@@ -809,26 +832,16 @@ class Mt5ExecutionEngine:
             elif signal not in NON_EXEC_RETCODES:
                 logger.info(info)
 
+        signal_handler = None
         if signal in EXIT_SIGNAL_ACTIONS:
             self._handle_exit_signals(signal, symbol, id, trade, sigmsg, comment)
         elif signal in BUYS:
-            self._handle_buy_signal(
-                signal,
-                symbol,
-                id,
-                trade,
-                price,
-                stoplimit,
-                buys,
-                sells,
-                sigmsg,
-                msg,
-                tfmsg,
-                riskmsg,
-                comment,
-            )
+            signal_handler = self._handle_buy_signal
         elif signal in SELLS:
-            self._handle_sell_signal(
+            signal_handler = self._handle_sell_signal
+        
+        if signal_handler is not None:
+            signal_handler(
                 signal,
                 symbol,
                 id,
@@ -866,51 +879,87 @@ class Mt5ExecutionEngine:
                     f"(e.g., if time_frame is 15m, iter_time must be 1.5, 3, 5, 15 etc)"
                 )
 
-    def _handle_signals(self, today, signals, buys, sells):
+    def _handle_one_signal(self, signal, today, buys, sells):
         try:
-            check_mt5_connection(**self.kwargs)
-            for signal in signals:
-                symbol = signal.symbol
-                trade: Trade = self.trades_instances[symbol]
-                if trade.trading_time() and today in self.trading_days:
-                    if signal.action is not None:
-                        action = (
-                            signal.action.value
-                            if isinstance(signal.action, TradeAction)
-                            else signal.action
-                        )
-                        self._run_trade_algorithm(
-                            action,
-                            symbol,
-                            signal.id,
-                            trade,
-                            signal.price,
-                            signal.stoplimit,
-                            buys,
-                            sells,
-                            signal.comment or self.comment,
+            symbol = signal.symbol
+            trade: Trade = self.trades_instances[symbol]
+            if trade.trading_time() and today in self.trading_days:
+                if signal.action is not None:
+                    action = (
+                        signal.action.value
+                        if isinstance(signal.action, TradeAction)
+                        else signal.action
+                    )
+                    self._run_trade_algorithm(
+                        action,
+                        symbol,
+                        signal.id,
+                        trade,
+                        signal.price,
+                        signal.stoplimit,
+                        buys,
+                        sells,
+                        signal.comment or self.comment,
+                    )
+            else:
+                if len(self.symbols) >= 10:
+                    if symbol == self.symbols[-1]:
+                        logger.info(
+                            f"Not trading Time !!!, STRATEGY={self.STRATEGY} , ACCOUNT={self.ACCOUNT}"
                         )
                 else:
-                    if len(self.symbols) >= 10:
-                        if symbol == self.symbols[-1]:
-                            logger.info(
-                                f"Not trading Time !!!, STRATEGY={self.STRATEGY} , ACCOUNT={self.ACCOUNT}"
-                            )
-                    else:
-                        logger.info(
-                            f"Not trading Time !!! SYMBOL={trade.symbol}, STRATEGY={self.STRATEGY} , ACCOUNT={self.ACCOUNT}"
-                        )
-                    self._check(buys[symbol], sells[symbol], symbol)
+                    logger.info(
+                        f"Not trading Time !!! SYMBOL={trade.symbol}, STRATEGY={self.STRATEGY} , ACCOUNT={self.ACCOUNT}"
+                    )
+                self._check(buys[symbol], sells[symbol], symbol)
 
         except Exception as e:
-            msg = f"Handling Signals, SYMBOL={symbol}, STRATEGY={self.STRATEGY} , ACCOUNT={self.ACCOUNT}"
+            msg = (
+                f"Error handling signal for SYMBOL={signal.symbol} (SIGNAL: {action}), "
+                f"STRATEGY={self.STRATEGY}, ACCOUNT={self.ACCOUNT}"
+            )
             self._print_exc(msg, e)
-            pass
+
+    def _handle_all_signals(self, today, signals, buys, sells, max_workers=50):
+        try:
+            check_mt5_connection(**self.kwargs)
+        except Exception as e:
+            msg = "Initial MT5 connection check failed. Aborting signal processing."
+            self._print_exc(msg, e)
+            return
+
+        if not signals:
+            return
+
+        # We want to create a temporary function that
+        # already has the 'today', 'buys', and 'sells' arguments filled in.
+        # This is necessary because executor.map only iterates over one sequence (signals).
+        signal_processor = functools.partial(
+            self._handle_one_signal, today=today, buys=buys, sells=sells
+        )
+        if self.multithread:
+            with concurrent.futures.ThreadPoolExecutor(
+                max_workers=max_workers
+            ) as executor:
+                # 'map' will apply our worker function to every item in the 'signals' list.
+                # It will automatically manage the distribution of tasks to the worker threads.
+                # We wrap it in list() to ensure all tasks are complete before moving on.
+                list(executor.map(signal_processor, signals))
+        else:
+            for signal in signals:
+                try:
+                    signal_processor(signal)
+                except Exception as e:
+                    self._print_exc(f"Failed to process signal {signal}: ", e)
 
     def _handle_period_end_actions(self, today):
         try:
             check_mt5_connection(**self.kwargs)
-            day_end = all(trade.days_end() for trade in self.trades_instances.values())
+            day_end = (
+                all(trade.days_end() for trade in self.trades_instances.values())
+                or (today in WEEK_ENDS and today != FRIDAY)
+                and self.period != "24/7"
+            )
             closing = self._is_closing()
             sessionmsg = f"{self.ACCOUNT} STARTING NEW TRADING SESSION ...\n"
             self._perform_period_end_actions(
@@ -924,10 +973,18 @@ class Mt5ExecutionEngine:
             self._print_exc(msg, e)
             pass
 
+    def _select_symbols(self):
+        for symbol in self.symbols:
+            if not MT5.symbol_select(symbol, True):
+                logger.error(
+                    f"Failed to select symbol {symbol} error = {MT5.last_error()}"
+                )
+
     def run(self):
-        while self._running:
+        while self._running and not self.shutdown_event.is_set():
             try:
                 check_mt5_connection(**self.kwargs)
+                self._select_symbols()
                 positions_orders = self._check_positions_orders()
                 if self.show_positions_orders:
                     self._display_positions_orders(positions_orders)
@@ -943,35 +1000,51 @@ class Mt5ExecutionEngine:
                         self._check(buys[symbol], sells[symbol], symbol)
                 else:
                     self._update_risk(weights)
-                    self._handle_signals(today, signals, buys, sells)
+                    self._handle_all_signals(today, signals, buys, sells)
                 self._sleep()
                 self._handle_period_end_actions(today)
             except KeyboardInterrupt:
-                logger.info(
-                    f"Stopping Execution Engine for {self.STRATEGY} STRATEGY on {self.ACCOUNT} Account"
-                )
-                break
+                self.stop()
+                sys.exit(0)
             except Exception as e:
                 msg = f"Running Execution Engine, STRATEGY={self.STRATEGY} , ACCOUNT={self.ACCOUNT}"
                 self._print_exc(msg, e)
+                self._sleep()
 
     def stop(self):
         """Stops the execution engine."""
-        self._running = False
-        logger.info(
-            f"Stopping Execution Engine for {self.STRATEGY} STRATEGY on {self.ACCOUNT} Account"
-        )
+        if self._running:
+            logger.info(
+                f"Stopping Execution Engine for {self.STRATEGY} STRATEGY on {self.ACCOUNT} Account"
+            )
+            self._running = False
+            self.shutdown_event.set()
         logger.info("Execution Engine stopped successfully.")
 
 
 def RunMt5Engine(account_id: str, **kwargs):
-    """Starts an MT5 execution engine for a given account.
-    Args:
-        account_id: Account ID to run the execution engine on.
-        **kwargs: Additional keyword arguments
-            _ symbol_list : List of symbols to trade.
-            - trades_instances : Dictionary of Trade instances.
-            - strategy_cls : Strategy class to use for trading.
+    """
+    Start an MT5 execution engine for a given account.
+
+    Parameters
+    ----------
+    account_id : str
+        Account ID to run the execution engine on.
+
+    **kwargs : dict
+        Additional keyword arguments. Possible keys include:
+
+        * symbol_list : list
+            List of symbols to trade.
+        * trades_instances : dict
+            Dictionary of Trade instances.
+        * strategy_cls : class
+            Strategy class to use for trading.
+
+    Returns
+    -------
+    None
+        Initializes and runs the MT5 execution engine.
     """
     log.info(f"Starting execution engine for {account_id}")
 
@@ -988,6 +1061,10 @@ def RunMt5Engine(account_id: str, **kwargs):
             symbol_list, trades_instances, strategy_cls, **kwargs
         )
         engine.run()
+    except KeyboardInterrupt:
+        log.info(f"Execution engine for {account_id} interrupted by user")
+        engine.stop()
+        sys.exit(0)
     except Exception as e:
         log.exception(f"Error running execution engine for {account_id}: {e}")
     finally:
@@ -1008,6 +1085,7 @@ def RunMt5Engines(accounts: Dict[str, Dict], start_delay: float = 1.0):
 
     for account_id, params in accounts.items():
         log.info(f"Starting process for {account_id}")
+        params["multithread"] = True
         process = mp.Process(target=RunMt5Engine, args=(account_id,), kwargs=params)
         process.start()
         processes[process] = account_id
